<?php
require_once(dirname(__FILE__) . '/../dashlet-helper.inc.php');

dashlets_data_check();

recent_alerts_dashlet();

function recent_alerts_dashlet() {
	
	$max_alerts = 15;

	$recent_alerts = array();

	$servers = get_servers();
	foreach ($servers as $server) {

		$alert_list = get_server_polled_key($server, 'alert_list');
		$alert_list = array_map('alert_list_to_human_readable', $alert_list);

		if (count($alert_list) > 0) {
			foreach ($alert_list as $index => $alerts) {
				if (isset($alerts['alert_list'])) {
					foreach ($alerts['alert_list'] as $alert) {

						if (isset($alert['time'])) {
                            $alert['server_id'] = $server['server_id'];
							$alert['server_name'] = $server['name'];
							$alert['time'] = strtotime($alert['time']);

							$recent_alerts[] = $alert;
						}
					}
				}
			}
		}
	}

	usort($recent_alerts, 'sort_alerts_by_time');
	if (count($recent_alerts) > $max_alerts)
	 	$recent_alerts = array_slice($recent_alerts, 0, $max_alerts);

     echo '
            <style>
            table.alerts th { cursor: pointer; }
            </style>
            <script type="text/javascript">
            $(function() {
                $("table.alerts").tablesorter({ });
            });
            </script>
            <table class="tactical alerts table table-no-margin table-condensed table-striped table-bordered host_status_summary_dashlet borderless service_status_summary_dashlet borderless">
                <thead>
                    <tr>
                        <th>' . _('Server') . '</th>
                        <th>' . _('Time') . '</th>
                        <th>' . _('Host') . '</th>
                        <th>' . _('Service') . '</th>
                        <th>' . _('State') . '</th>
                        <th>' . _('State Type') . '</th>
                        <th>' . _('Output') . '</th>
                    </tr>
                </thead>
                <tbody>';

	if (count($recent_alerts) > 0) {
		foreach ($recent_alerts as $alert) {
			recent_alerts_dashlet_build_row($alert);
		}
	} else {
        echo '<tr><td colspan="7" style="text-align: center;">' . _('You have no recent alerts!') . '</td></tr>';
    }

    echo '</tbody></table>';
}

function recent_alerts_dashlet_build_row($alert) {

	// we need all this
	if (   !isset($alert['time']) 
        || !isset($alert['server_id'])
		|| !isset($alert['server_name']) 
		|| !isset($alert['host']) 
		|| !isset($alert['state']) 
		|| !isset($alert['state_type']) 
		|| !isset($alert['type'])
		|| !isset($alert['output']))
		return;

	$alert['time'] = date('Y-m-d H:i:s', $alert['time']);

	if (empty($alert['service']))
		$alert['service'] = '';

	$state_class = '';
	switch ($alert['type']) {
        case NSLOG_HOST_UP:
            $state_class = 'havehostup';
            break;

        case NSLOG_HOST_DOWN:
            $state_class = 'havehostdown';
            break;
        
        case NSLOG_HOST_UNREACHABLE:
            $state_class = 'havehostunreachable';
            break;
        
        case NSLOG_SERVICE_OK:
            $state_class = 'haveserviceok';
            break;
        
        case NSLOG_SERVICE_WARNING:
            $state_class = 'haveservicewarning';
            break;
        
        case NSLOG_SERVICE_CRITICAL:
            $state_class = 'haveservicecritical';
            break;
        
        case NSLOG_SERVICE_UNKNOWN:
            $state_class = 'haveserviceunknown';
            break;
    }

    if (!empty($state_class))
    	$state_class = ' class="' . $state_class . '"';

    $server_link = get_server_base_url($alert['server_id'], $core_include_cgi = false);
    $server_link = '<a href="' . $server_link . '" target="_blank">' . $alert['server_name'] . '</a>';

    $host_link = get_tactical_link($alert['host'], $alert['server_id'], 
    "includes/components/xicore/status.php?show=hostdetail&host=" . urlencode($alert['host']),
    "extinfo.cgi?type=1&host=" . urlencode($alert['host']));

    $service_link = '';
    if (!empty($alert['service']))
        $service_link = get_tactical_link($alert['service'], $alert['server_id'],
            "includes/components/xicore/status.php?show=servicedetail&dest=auto&host=" . urlencode($alert['host']) . "&service=" . urlencode($alert['service']),
            "extinfo.cgi?type=2&host=" . urlencode($alert['host']) . "&service=" . urlencode($alert['service']));

	echo '
		<tr>
			<td>' . $server_link . '</td>
			<td>' . $alert['time'] . '</td>
			<td>' . $host_link . '</td>
			<td>' . $service_link . '</td>
			<td' . $state_class . '>' . $alert['state'] . '</td>
			<td>' . $alert['state_type'] . '</td>
			<td>' . $alert['output'] . '</td>
		</tr>';
}

function sort_alerts_by_time($a, $b) {

	if (!empty($a['time']) && !empty($b['time']))
		return $a['time'] < $b['time'];

	return false;
}
