#!/bin/bash -e

. ./fusion-sys.cfg
. ./functions.sh

# Check if user already configured repos
if [ -f installed.repos ]; then
    echo "Repos already configured - exiting."
    exit 0
fi
echo "Configuring Repos..."

# Check distro is CentOS / RHEL
case "$dist" in
    el6 | el7 )
        :
        ;;
    ubuntu14 | ubuntu16 | ubuntu18 )
        :
        ;;
    debian8 | debian9 )
        :
        ;;
    el5 )
        echo "CentOS / RHEL 5 is no longer supported due to end of life."
        echo "Please use CentOS / RHEL 6 or 7 instead."
        exit 1
        ;;
    *)
        echo "$dist is not currently supported. Please use one of the following distros:" >&2
        echo "  CentOS, RHEL, Oracle 6 or 7" >&2
        echo "  Ubuntu 14, 16, or 18 LTS" >&2
        echo "  Debian 8 or 9" >&2
        exit 1
esac

# Check that the Red Hat installation is valid
if [ "$distro" = "RedHatEnterpriseServer" ] && [ -x /usr/sbin/rhn_check ] && ! /usr/sbin/rhn_check >/dev/null 2>&1; then
    ## Fails initial check with newer subscription method, nested if to check for subscription and proceed correctly if found -SR
    if [ -x /usr/bin/subscription-manager ] && [[ -z `subscription-manager list | grep Status: | grep -qF Subscribed` ]]; then
        echo "Passed New Subscription Check"
    else
        echo "Your Red Hat Installation is not registered or does not have proper entitlements." >&2
        echo "Please register or enable entitlements at rhn.redhat.com." >&2
        exit 1;
    fi
fi

# Make sure everything needed is still present, and we should be working 
# with a current system anyway

if [ "$distro" == "Ubuntu" ]; then 
    #./ubuntumods
    echo "Repos configured OK"
    touch installed.repos
    exit 0
fi

if [ "$distro" == "Debian" ]; then
    #./debianmods
    echo "Repos configured OK"
    touch installed.repos
    exit 0
fi

if [ "$distro" == "CentOS" ] || [ "$distro" == "RedHatEnterpriseServer" ]; then
    if [ "$1" != noupdate ]; then
        echo "Updating system..."
        yum -y update
    fi
fi

echo "Repo configured OK"
touch installed.repos