#!/bin/bash -e

. ./fusion-sys.cfg

# Check whether we have sufficient privileges
if [ $(id -u) -ne 0 ]; then
    echo "This script needs to be run as root/superuser." >&2
    exit 1
fi

# Was previous step completed?
if [ ! -f installed.repos ]; then
    echo "Repos were not properly prepared - run previous script" >&2
    exit 1
fi

# Are prereqs already installed?
if [ -f installed.prereqs ]; then
    echo "Prereqs already installed - skipping..."
    echo "Note: If you want to update the RPMs installed on this system, run your package manager's update command"
    exit 0
fi

# =======================
# Install Prereqs
# -----------------------

# Run basic yum install for now
unsupported="false"
if [ "$distro" == "RedHatEnterpriseServer" ] || [ "$distro" == "CentOS" ] || [ "$distro" == "OracleLinux" ] || [ "$distro" == "CloudLinux" ]; then
	if [ "$dist" == "el6" ]; then
        yum install httpd mysql mysql-server php php-pdo php-devel php-mysql php-mbstring php-ldap zip unzip ntp sysstat -y
        yum install -y "@Japanese Support" "@Chinese Support" "@Korean Support";
	elif [ "$dist" == "el7" ]; then
		yum install httpd mysql mariadb mariadb-server php php-pdo php-devel php-mysql php-mbstring php-ldap zip unzip ntp sysstat -y   
	else
        unsupported="true"
	fi
elif [ "$dist" == "ubuntu14" ]; then
    apt-get update
    # Set DEBIAN_FRONTEND to noninteractive to avoid the MySQL graphical password prompt
    DEBIAN_FRONTEND=noninteractive apt-get install -y apache2 libapache2-mod-php5 mysql-server php5 php5-mysql php5-curl php5-ldap php5-dev zip unzip ntp sysstat
elif [ "$dist" == "ubuntu16" ] || [ "$dist" == "ubuntu18" ]; then
    apt-get update
    DEBIAN_FRONTEND=noninteractive apt-get install apache2 libapache2-mod-php mysql-server php php-mysql php-pdo php-curl php-xml php-dev php-mbstring php-ldap zip unzip ntp sysstat -y
elif [ "$dist" == "debian8" ]; then
    apt-get update
    DEBIAN_FRONTEND=noninteractive apt-get install -y apache2 libapache2-mod-php5 mysql-server php5 php5-mysql php5-curl php5-dev php5-ldap zip unzip ntp sysstat uuid-runtime sudo
elif [ "$dist" == "debian9" ]; then
    apt-get update
    DEBIAN_FRONTEND=noninteractive apt-get install -y apache2 libapache2-mod-php mysql-server php php-mysql php-dev php-xml php-curl php-ldap zip unzip ntp sysstat uuid-runtime sudo net-tools
else
    unsupported="true"
fi

if [ "$unsupported" = "true" ]; then
    echo "Operating system not supported. Supported OS are CentOS/RHEL 6 or 7, Ubuntu 14/16 LTS, and Debian 7,8 or 9."
fi

echo "Prerequisites installed OK"
touch installed.prereqs